@@ -72,13 +72,13 @@ server_constructor (Server              * server,
 	server->username = NULL;
 	server->password = NULL;
 	server->port = 25;
-	server->max_connections = 1;
+	server->max_connections = 2;
 	server->idle_secs_before_timeout = 180;
 	server->last_newgroup_list_time = 1; /* Typhoon can't handle 0 */
 	server->_groups_dirty = 0;
 	server->_groups_loaded = 0;
 	server->_groups = g_ptr_array_new ();
-	server->_group_memchunk = g_mem_chunk_create (Group, 100, G_ALLOC_ONLY);
+	server->_group_memchunk = memchunk_new (sizeof(Group), 100, FALSE);
 }
 
 void
@@ -92,7 +92,7 @@ server_destructor (PanObject *obj)
 	for (i=0; i!=server->_groups->len; ++i)
 		group_destructor (GROUP(g_ptr_array_index(server->_groups,i)));
 	g_ptr_array_free (server->_groups, TRUE);
-	g_mem_chunk_destroy (server->_group_memchunk);
+	memchunk_destroy (server->_group_memchunk);
 	server->_groups = NULL;
 	server->_group_memchunk = NULL;
 
@@ -121,12 +121,6 @@ server_new (void)
 ****
 ***/
 
-static gchar*
-load_groups_describe (const StatusItem * item)
-{
-	return g_strdup (_("Loading Groups"));
-}
-
 ServerGroupsType
 server_get_group_type (const Group * group)
 {
@@ -171,7 +165,7 @@ server_ensure_groups_loaded (Server * server, ServerGroupsType type)
 	debug_enter ("server_ensure_groups_loaded");
 
 	/* sanity clause */
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (type!=0);
 
 	/* if we're going to load groups, start up a statusitem */
@@ -179,17 +173,16 @@ server_ensure_groups_loaded (Server * server, ServerGroupsType type)
 	if (load_needed)
 	{
 		ServerGroupsType dirty;
+		ServerGroupsType types_to_load;
 		
-		StatusItem * status = STATUS_ITEM(status_item_new(load_groups_describe));
-		status_item_set_active (status, TRUE); 
-
+		/* only load the ones we haven't already loaded */
+		types_to_load = type & ~server->_groups_loaded;
 		server->_groups_loaded |= type;
-		dirty = server->_groups_dirty;
-		file_grouplist_load (server, type, status);
-		server->_groups_dirty = dirty;
 
-		status_item_set_active (status, FALSE); 
-		pan_object_unref (PAN_OBJECT(status));
+		/* load whatever needs loading */
+		dirty = server->_groups_dirty;
+		file_grouplist_load (server, types_to_load, NULL);
+		server->_groups_dirty = dirty;
 	}
 
 	debug_exit ("server_ensure_groups_loaded");
@@ -202,7 +195,7 @@ server_get_groups (Server * server, ServerGroupsType type)
 	GPtrArray * a = g_ptr_array_new ();
 	debug_enter ("server_get_groups");
 
-	g_return_val_if_fail (server!=NULL, a);
+	g_return_val_if_fail (server_is_valid(server), a);
 	g_return_val_if_fail (type!=0, a);
 
 	/* make sure all the needed groups are loaded */
@@ -231,7 +224,7 @@ server_set_group_type_dirty (Server * server, ServerGroupsType type)
 	debug_enter ("server_set_group_type_dirty");
 
 	/* sanity clause */
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (type!=0);
 
 	/* mark dirty */
@@ -244,7 +237,7 @@ server_set_group_type_dirty (Server * server, ServerGroupsType type)
 void
 server_save_grouplist_if_dirty (Server * server, StatusItem * status)
 {
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (server_saving_enabled);
 
 	if (server->_groups_dirty)
@@ -260,7 +253,7 @@ server_save_if_dirty (Server * server, StatusItem * status)
 	guint i;
 	debug_enter ("server_save_if_dirty");
 
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 
 	server_save_grouplist_if_dirty (server, status);
 
@@ -286,7 +279,7 @@ void
 server_set_online (Server    * server,
                    gboolean    online)
 {
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	server->online = online ? 1 : 0;
 
 	fire_online_status_changed (server, server->online);
@@ -305,10 +298,9 @@ server_alloc_new_group (Server * server)
 {
 	Group * group;
 
-        g_return_val_if_fail (server!=NULL, NULL);
-        g_return_val_if_fail (server->_group_memchunk!=NULL, NULL);
+        g_return_val_if_fail (server_is_valid (server), NULL);
 
-	group =  g_chunk_new0 (Group, server->_group_memchunk);
+	group = (Group*) memchunk_alloc (server->_group_memchunk);
 	group->server = server;
 
 	return group;
@@ -333,20 +325,21 @@ compare_ppgroup_ppgroup_name (const void * a, const void * b)
 }
 
 
-void
-server_add_groups (Server       * server,
-                   Group       ** groups,
-                   gint           group_qty,
-                   GPtrArray    * fillme_used,
-                   GPtrArray    * fillme_not_used)
+static void
+server_add_groups_impl (Server       * server,
+                        Group       ** groups,
+                        gint           group_qty,
+                        GPtrArray    * fillme_used,
+                        GPtrArray    * fillme_not_used,
+                        gboolean       initializing)
 {
 	gint i;
 	GHashTable * old;
 	GPtrArray * added;
-	debug_enter ("server_add_groups");
+	debug_enter ("server_add_groups_impl");
 
 	/* entry assertions */
-	g_return_if_fail (server != NULL);
+	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (groups != NULL);
 	g_return_if_fail (group_qty >= 1);
 
@@ -403,27 +396,49 @@ server_add_groups (Server       * server,
 		       sizeof(gpointer),
 		       compare_ppgroup_ppgroup_name);
 
-		/* mark the appropriate sets as dirty */
-		type = server_get_groups_type ((const Group**)added->pdata, added->len);
-		server_set_group_type_dirty (server, type);
+		if (!initializing)
+		{
+			/* mark the appropriate sets as dirty */
+			type = server_get_groups_type ((const Group**)added->pdata, added->len);
+			server_set_group_type_dirty (server, type);
 
-		/* let listeners know which were added */
-		fire_groups_added (server, (Group**)added->pdata, added->len);
+			/* let listeners know which were added */
+			fire_groups_added (server, (Group**)added->pdata, added->len);
 
-		/* let the caller know which were added */
-		if (fillme_used != NULL)
-			pan_g_ptr_array_assign (fillme_used, added->pdata, added->len);
+			/* let the caller know which were added */
+			if (fillme_used != NULL)
+				pan_g_ptr_array_assign (fillme_used, added->pdata, added->len);
+
+			/* save them for safe-keeping */
+			server_save_grouplist_if_dirty (server, NULL);
+		}
 	}
 
-	/* save them for safe-keeping */
-	server_save_grouplist_if_dirty (server, NULL);
-
 	/* cleanup */
 	g_hash_table_destroy (old);
 	g_ptr_array_free (added, TRUE);
-	debug_exit ("server_add_groups");
+	debug_exit ("server_add_groups_impl");
 }
 
+void
+server_add_groups (Server       * server,
+                   Group       ** groups,
+                   gint           group_qty,
+                   GPtrArray    * fillme_used,
+                   GPtrArray    * fillme_not_used)
+{
+	server_add_groups_impl (server, groups, group_qty, fillme_used, fillme_not_used, FALSE);
+}
+
+void
+server_init_groups (Server       * server,
+                    Group       ** groups,
+                    gint           group_qty,
+                    GPtrArray    * fillme_used,
+                    GPtrArray    * fillme_not_used)
+{
+	server_add_groups_impl (server, groups, group_qty, fillme_used, fillme_not_used, TRUE);
+}
 
 void
 server_remove_groups (Server * server, Group ** groups, int qty)
@@ -434,7 +449,7 @@ server_remove_groups (Server * server, Group ** groups, int qty)
 	debug_enter ("server_remove_groups");
 
 	/* sanity clause */
-	g_return_if_fail (server != NULL);
+	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (qty > 0);
 	for (i=0; i<qty; ++i) {
 		g_return_if_fail (groups[i] != NULL);
@@ -490,7 +505,7 @@ server_get_named_group_impl (Server           * server,
 	debug_enter ("server_get_named_group_impl");
 
 	/* sanity clause */
-	g_return_val_if_fail (server!=NULL, NULL);
+	g_return_val_if_fail (server_is_valid (server), NULL);
 	g_return_val_if_fail (is_nonempty_string(name), NULL);
 	g_return_val_if_fail (set!=0, NULL);
 
@@ -542,8 +557,26 @@ server_has_group (const Server  * server,
 	return server_has_group_in_type (server, name, SERVER_GROUPS_ALL);
 }
 
+gboolean
+server_is_valid (const Server * server)
+{
+	g_return_val_if_fail (server!=NULL, FALSE);
+	g_return_val_if_fail (is_nonempty_string (server->name), FALSE);
+	g_return_val_if_fail (server->_group_memchunk!=NULL, FALSE);
+	g_return_val_if_fail (server->_groups!=NULL, FALSE);
+	/* arbitrary fishing for corruption */
+	g_return_val_if_fail ((server->_groups_dirty & ~SERVER_GROUPS_ALL) == 0, FALSE);
+	g_return_val_if_fail ((server->_groups_loaded & ~SERVER_GROUPS_ALL) == 0, FALSE);
 
+	return TRUE;
+}
 
+const gchar * server_get_name (const Server * server)
+{
+	g_return_val_if_fail (server_is_valid (server), "");
+
+	return server->name;
+}
 
 void
 server_destroy_groups (Server * server, struct _Group ** groups, gint qty)
@@ -590,7 +623,7 @@ fire_groups_removed (Server * server, Group ** groups, gint group_qty)
 {
 	GPtrArray * a;
 
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (groups!=NULL);
 	g_return_if_fail (group_qty>0);
 
@@ -617,7 +650,7 @@ fire_groups_added (Server * server, Group ** groups, gint group_qty)
 {
 	GPtrArray * a;
 
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid(server));
 	g_return_if_fail (groups!=NULL);
 	g_return_if_fail (group_qty>0);
 
@@ -642,7 +675,7 @@ server_get_online_status_changed_callback (void)
 static void
 fire_online_status_changed (Server * server, gboolean online)
 {
-	g_return_if_fail (server!=NULL);
+	g_return_if_fail (server_is_valid (server));
 
 	pan_callback_call (server_get_online_status_changed_callback(),
 	                   server,
