@@ -108,17 +108,17 @@ dialog_about (GtkWidget * window)
 static gchar *
 get_credits_translators (void)
 {
-	return g_strdup ("\n\tTakeshi Aihana \n\tJuan Carlos Castro y Castro \n\tGrkem etin \n\tTov Are Jacobsen \n\tTivo Leedjrv \n\tDuarte Loreto \n\tChristian Meyer \n\tChristian Rose \n\tGNOME PL Team \n\tMarcel Telka \n\tAlmer S. Tigelaar \n\tSimos Xenitellis \n\tZipeco");
+	return g_strdup ("\n\tT. Aihana\n\tDuarte Loreto\n\tChristian Rose\n\tMarcel Telka\n\tReinhout van Schouwen");
 }
 
 static gchar *
 get_credits_code (void)
 {
-	return g_strdup ("\n\tKari Hameenaho \n\tBas Mevissen \n\tMarc Prud'hommeaux \n\tThe Samba Team for SPA authentication code \n\tChristian Suder");
+	return g_strdup ("\n\tGabi Davar\n\tJiri Kubicek\n\tColin Leroy\n\tSam Solon\n\tJeff Stedfast");
 }
 
 static gchar *
 get_credits_users (void)
 {
-	return g_strdup ("\n\tLars Bamberger \n\tDouglas Bollinger \n\tDevin Carraway \n\tGabi Davar \n\tLaurent Duperval \n\tAlex Evonosky \n\tChris Gayler \n\tRoger Gregory \n\tErik W Hanson \n\tDan Hensley \n\tJrg Hoh \n\tCarl Hudkins \n\tJohn J. LeMay Jr.  \n\tJeff Lightfoot \n\tChristian Lohmaier \n\tAvtar Marwaha \n\tBrandon McCombs \n\tJohn Kapp \n\tKrishna Naik \n\tTorbjorn Olofsson \n\tChris Petersen \n\tJean Ribiere \n\tJukka Salmi \n\tT'Az \n\tStefan Van Theemsche \n\tJoseph Turian \n\tFrank Van Damme \n\tMart van de Wege \n\tAlen Williams \n\tAlan Young \n\tRon KA4INM Youvan");
+	return g_strdup ("\n\tErik Anderson\n\tNed Boddie\n\tMike Brodbelt\n\tHaines Brown\n\tMartin Bruchanov\n\tAric S. Campling\n\tBill Cattell\n\tJeff E.\n\tHimanshu J. Gohel \n\tAurelien Gouny\n\tJim Henderson\n\tBill Huang\n\tJohn Levon\n\tMirko Liss\n\tMichael McIntyre\n\tJohn Mellor\n\tSven Neuhaus\n\tMr. P\n\tCraig Orsinger\n\tWilliam Peterson\n\tJean Ribiere\n\tKerstin Rohlfs\n\tJoshua Rosen\n\tJason Rumney\n\tJorge Salinas\n\tFlorian Schmidt\n\tT'aZ\n\tPascal Valois");
 }
