@@ -27,8 +27,10 @@
 #include <libgnomeui/libgnomeui.h>
 
 #include <pan/base/debug.h>
+#include <pan/base/run.h>
 #include <pan/base/pan-i18n.h>
 #include <pan/base/pan-glib-extensions.h>
+#include <pan/base/util-mime.h>
 
 #include <pan/globals.h>
 #include <pan/gui-headers.h>
@@ -101,8 +103,15 @@ add_header_nolock (GtkWidget        * table,
        	w1 = gtk_label_new (str1);
 
 	breakpoint = find_breakpoint (str2, delimiter, max_width);
-	if (breakpoint != -1)
+	if (breakpoint == -1)
 	{
+		/* string is short; show it all */
+		w2 = gtk_label_new (str2);
+		gtk_misc_set_alignment (GTK_MISC(w2), 0.0, 0.5);
+	}
+	else
+	{
+		/* string is long, so truncate and make a "<More>" button */
 		GtkWidget * hbox = gtk_hbox_new (FALSE, 0);
 		GtkWidget * l = NULL;
 		GtkWidget * l2 = gtk_label_new (_("<More>"));
@@ -110,10 +119,9 @@ add_header_nolock (GtkWidget        * table,
 		gchar * pch = NULL;
 
 		/* create the truncated label */
-		pch = g_strdup(str2);
+		pch = pan_strdup_alloca (str2);
 		pch[breakpoint] = '\0';
 		l = gtk_label_new (pch);
-		g_free (pch);
 
 		/* add them both to the button */
 		gtk_box_pack_start(GTK_BOX(hbox), l, FALSE, FALSE, 0);
@@ -142,12 +150,6 @@ add_header_nolock (GtkWidget        * table,
 			GTK_OBJECT(w2), "clicked", showmore_cb, NULL);
 		gtk_tooltips_set_tip (GTK_TOOLTIPS(ttips), w2, str2, NULL);
 	}
-	else
-	{
-		/* string is short; show it all */
-		w2 = gtk_label_new  (str2);
-		gtk_misc_set_alignment (GTK_MISC(w2), 0.0, 0.5);
-	}
 
 	gtk_misc_set_alignment (GTK_MISC(w1), 1.0, 0.5);
 
@@ -161,99 +163,139 @@ add_header_nolock (GtkWidget        * table,
 	debug_exit ("add_header_nolock");
 }
 
+static const Run*
+get_header_run (GHashTable * hash, const gchar * header)
+{
+	Run key;
+	(void) run_init (&key, header, strlen(header));
+	return (const Run *) g_hash_table_lookup (hash, &key);
+}
+
+static void
+get_and_add_header_nolock (GtkWidget    * table,
+                           gint         * row,
+                           GHashTable   * run_hash,
+                           const gchar  * header,
+                           const gchar  * header_i18n,
+                           gchar          delimiter,
+                           gint           max_len)
+{
+	const Run * val = get_header_run (run_hash, header);
+	if (val != NULL)
+	{
+		gchar * decoded = NULL;
+		gchar * pch;
+
+		pan_strndup_alloca (pch, val->str, val->len);
+
+		if (pan_header_is_8bit_encoded (pch)) 
+			pch = decoded = g_mime_utils_8bit_header_decode (pch);
+
+		add_header_nolock (table, (*row)++, header_i18n, pch, delimiter, max_header_width);
+
+		g_free (decoded);
+	}
+}
+
 void
-gui_headers_set_nolock (GtkWidget         * table,
-		        const Article     * article,
-		        gulong              header_fields)
+gui_headers_set_nolock (GtkWidget       * table,
+		        const Article   * article,
+		        gulong            header_fields)
 {
-	const gchar * newsgroups;
-	const gchar * pch;
 	gint row = 0;
-	debug_enter ("gui_headers_set");
+	gpointer buf = NULL;
+	gchar * message = NULL;
+	GHashTable * hash = NULL;
+	debug_enter ("gui_headers_set_nolock");
 
-	g_return_if_fail (table != NULL);
-
-	/* remove children */
-	gtk_container_foreach (
-		GTK_CONTAINER (table),
-		GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
+	/* remove previous children */
+	g_return_if_fail (GTK_IS_TABLE(table));
+	gtk_container_foreach (GTK_CONTAINER (table),
+	                       GTK_SIGNAL_FUNC (gtk_widget_destroy), NULL);
 
 	/* if no article, that's all we need to do */
 	if (article == NULL)
 		return;
 
-	/* if author turned on, show it unconditionally */
-	if (header_fields & UI_HEADER_AUTHOR)
+	/* get the article headers */
+	message = article_get_message (article);
+	if (article_get_header_run_hash_from_text (message, &hash, &buf))
 	{
-		gchar * author = article_get_author_str (article);
-		add_header_nolock (table, row++, _("From: "), author, ' ', max_header_width);
-		g_free (author);
+		const Run * val;
+		const Run * val2;
+		gchar * pch;
+
+		/* if author turned on, show it unconditionally */
+		if (header_fields & UI_HEADER_AUTHOR)
+			get_and_add_header_nolock (table, &row, hash, HEADER_FROM, _("From: "), ' ', max_header_width);
+
+		/* if newsgroups turned on, and this is a crosspost, show it */
+		if (header_fields & UI_HEADER_NEWSGROUPS) {
+			val = get_header_run (hash, HEADER_NEWSGROUPS);
+			if (val!= NULL && run_strchr(val,',')!=NULL) {
+				pan_strndup_alloca (pch, val->str, val->len);
+				add_header_nolock (table, row++, _("Newsgroups: "), pch, ',', max_header_width);
+			}
+		}
+
+		/* message-id */
+		if (header_fields & UI_HEADER_MESSAGE_ID)
+			get_and_add_header_nolock (table, &row, hash, HEADER_MESSAGE_ID, _("Message-Id: "), ' ', max_header_width);
+
+		/* references */
+		if (header_fields & UI_HEADER_REFERENCES)
+			get_and_add_header_nolock (table, &row, hash, HEADER_REFERENCES, _("References: "), ' ', max_header_width);
+
+		/* date */
+		if (header_fields & UI_HEADER_DATE)
+			get_and_add_header_nolock (table, &row, hash, HEADER_DATE, _("Date: "), '\0', max_header_width);
+
+		/* reply-to */
+		if (header_fields & UI_HEADER_REPLY_TO) {
+			val = get_header_run (hash, HEADER_REPLY_TO);
+			if (val!=NULL && run_strcmp(val, article->author_addr)) {
+				pan_strndup_alloca (pch, val->str, val->len);
+				add_header_nolock (table, row++, _("Reply-To: "), pch, '\0', max_header_width);
+			}
+		}
+
+		/* if followups turned on, followup_to exists and != newsgroups, show */
+		if (header_fields & UI_HEADER_FOLLOWUP_TO) {
+			val = get_header_run (hash, HEADER_FOLLOWUP_TO);
+			if (val!=NULL) {
+				val2 = get_header_run (hash, HEADER_NEWSGROUPS);
+				if (run_runcmp (val, val2)) {
+					pan_strndup_alloca (pch, val->str, val->len);
+					add_header_nolock (table, row++, _("Followup-To: "), pch, ',', max_header_width);
+				}
+			}
+		}
+
+		/* newsreader */
+		if (header_fields & UI_HEADER_NEWSREADER) {
+			gint i;
+			static gchar * tags[] = { HEADER_USER_AGENT, HEADER_X_NEWSREADER, HEADER_X_MAILER };
+			static gint tag_count = sizeof tags / sizeof tags[0];
+			for (i=0; i<tag_count; ++i) {
+				if ((val = get_header_run (hash, tags[i]))) {
+					pan_strndup_alloca (pch, val->str, val->len);
+					add_header_nolock (table, row++, _("Newsreader:"), pch, ' ', max_header_width);
+					break;
+				}
+			}
+		}
+
+		/* subject */
+		if (header_fields & UI_HEADER_SUBJECT)
+			get_and_add_header_nolock (table, &row, hash, HEADER_SUBJECT, _("Subject: "), ' ', max_header_width);
+
+		g_hash_table_destroy (hash);
+		g_free (buf);
 	}
 
-	/* if newsgroups turned on, and this is a crosspost, show it */
-	newsgroups = pch = article_get_header (article, HEADER_NEWSGROUPS);
-	if ((header_fields & UI_HEADER_NEWSGROUPS)
-		&& is_nonempty_string (pch)
-		&& strchr (pch, ',')!=NULL)
-	{
-		add_header_nolock (table, row++, _("Newsgroups: "), pch, ',', max_header_width);
-	}
+	g_free (message);
 
-	/* if message-id turned on, show it unconditionally */
-	if (header_fields & UI_HEADER_MESSAGE_ID)
-	{
-		const gchar * s = article_get_message_id (article);
-		add_header_nolock (table, row++, _("Message-Id: "), s, '\0', max_header_width);
-	}
-
-	/* if references turned on, and references exist, show them */
-	if (header_fields & UI_HEADER_REFERENCES)
-	{
-		const gchar * s = article_get_header (article, HEADER_REFERENCES);
-		if (is_nonempty_string (s))
-			add_header_nolock (table, row++, _("References: "), s, ' ', max_header_width);
-	}
-
-	/* if date turned on, show it unconditionally. */
-	if (header_fields & UI_HEADER_DATE)
-	{
-		gchar buf[256];
-		get_date_display_string (article->date, body_date_format, buf, sizeof(buf));
-		add_header_nolock (table, row++, _("Date: "), buf, '\0', max_header_width);
-	}
-
-	/* if reply_to turned on, and reply_to exists and != author, show it */
-	pch = article_get_header (article, HEADER_REPLY_TO);
-	if ((header_fields & UI_HEADER_REPLY_TO)
-		&& is_nonempty_string (pch)
-		&& pan_strcmp (pch, article->author_addr))
-	{
-		add_header_nolock (table, row++, _("Reply-To: "), pch, '\0', max_header_width);
-	}
-
-	/* if followups turned on, followup_to exists and != newsgroups, show */
-	pch = article_get_header (article, HEADER_FOLLOWUP_TO);
-	if ((header_fields & UI_HEADER_FOLLOWUP_TO)
-		&& is_nonempty_string (pch)
-		&& pan_strcmp (pch, newsgroups))
-	{
-		add_header_nolock (table, row++, _("Followup-To: "), pch, ',', max_header_width);
-	}
-
-	/* if newsreader header found, display it */
-	if ((header_fields & UI_HEADER_NEWSREADER)
-		&& is_nonempty_string (pch = article_get_newsreader(article)))
-	{
-		add_header_nolock (table, row++, _("Newsreader:"), pch, ' ', max_header_width);
-		g_free (pch);
-	}
-
-	if (header_fields & UI_HEADER_SUBJECT)
-	{
-		add_header_nolock (table, row++, _("Subject: "), article_get_subject(article), ' ', max_header_width);
-	}
-
-	debug_exit ("gui_headers_set");
+	debug_exit ("gui_headers_set_nolock");
 }
 
 void
