@@ -4,6 +4,7 @@
 #include <string.h>
 
 #include <glib.h>
+#include <pan/base/base-prefs.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/util-wrap.h>
 
@@ -16,6 +17,16 @@ int main (void)
 	gchar * out;
 	const gchar * expected_out;
 
+	/* init pan */
+	if (1) {
+		char * base = getenv ("srcdir");
+		char * data_dir;
+		base = g_strdup_printf ("%s/", base!=NULL ? base : getenv("PWD"));
+		data_dir = g_strdup_printf ("%s/data/", base);
+		base_prefs_init (data_dir, g_get_tmp_dir(), ">", 100, FALSE, FALSE);
+		g_free (data_dir);
+	}
+
 	/* blank lines between quotes */
 	++num;
 	wrap_column = 50;
@@ -186,25 +197,26 @@ int main (void)
 
 	/* rot13 */
 	++num;
-	in = "Rot-13 started with rn, trn and similar newsreaders back in the mid-1980's.  It was common practice for a while for offending messages, and messages with some hint or disclosure (such as the answer to a question or puzzle posed in the message, or for covering spoilers to TV or movie episodes).";
-	out = rot13 (in);
+	in = g_strdup ("Rot-13 started with rn, trn and similar newsreaders back in the mid-1980's.  It was common practice for a while for offending messages, and messages with some hint or disclosure (such as the answer to a question or puzzle posed in the message, or for covering spoilers to TV or movie episodes).");
+	out = g_strdup (in);
+	rot13_inplace (out);
 	expected_out = "Ebg-13 fgnegrq jvgu ea, gea naq fvzvyne arjfernqref onpx va gur zvq-1980'f.  Vg jnf pbzzba cenpgvpr sbe n juvyr sbe bssraqvat zrffntrf, naq zrffntrf jvgu fbzr uvag be qvfpybfher (fhpu nf gur nafjre gb n dhrfgvba be chmmyr cbfrq va gur zrffntr, be sbe pbirevat fcbvyref gb GI be zbivr rcvfbqrf).";
 	if (strcmp(out,expected_out)) return num;
 	g_message ("PASS rot13");
 
 	/* rot13 2 */
 	++num;
-	replace_gstr (&out, rot13(out));
+	rot13_inplace (out);
 	if (strcmp(in,out)) return num;
 	replace_gstr (&out, NULL);
 	g_message ("PASS rot13 twice for double security");
 
 	/* rot13 3 */
 	++num;
-	in  = "here is a line with a ,, but the line should not be truncated.";
-	out = rot13 (in);
+	in = g_strdup ("here is a line with a ,, but the line should not be truncated.");
+	rot13_inplace (in);
 	expected_out = "urer vf n yvar jvgu n ,, ohg gur yvar fubhyq abg or gehapngrq.";
-	if (strcmp (out, expected_out)) return num;
+	if (strcmp (in, expected_out)) return num;
 	g_message ("PASS rot13 with , , or  characters");
 
 	/* mute quoted test 2 */
